//-----------------------------------
// Performance test program for CaTS
//  Input file:  performance_list
//  Output file: performance.html
//-----------------------------------

#include <list>
#include <stdlib.h>
#include <assert.h>
#include <stdio.h>
#include <time.h>
#include <string.h>
#include "testresult.h"

using namespace std;

//--------------------------------------------------------------
// Printer classes
//--------------------------------------------------------------
class printer
{
public:
  virtual void begin()=0;
  virtual void end()=0;
  virtual void beginTable()=0;
  virtual void endTable()=0;
  virtual void beginRow()=0;
  virtual void endRow()=0;
  virtual void beginEntry()=0;
  virtual void endEntry()=0;
  virtual void printString(const char *s)=0;
  virtual void hline()=0;
  virtual void flush()=0;
  virtual void newLine()=0;
  void entry(const char *s)
  {
    beginEntry();
    printString(s);
    endEntry();
  }
};

/* LaTeX printer not complete!
class LaTeXprinter:public printer
{
  FILE *f;
public:
  void begin()
  {
    f=fopen("ud.tex","w");

    fprintf(f,"\\documentclass[12pt,a4paper,dvips]{article}"
	    "\\usepackage[english]{babel}"
	    "\\usepackage{t1enc}"
	    "\\usepackage{a4}"
	    "\\usepackage{epsfig}"
	    "\\usepackage{amsfonts}"
	    "\\usepackage{latexsym}"
	    "\\begin{document}\n");
  }

  void end()
  {
    fprintf(f,"\\end{document}");

    fclose(f);
    system("latex ud.tex>/dev/null");
  }

  void beginTable()
  {
      fprintf(f,"\\begin{tabular}[t]{|l|rrrrr|rrrrrr|}\n");
      fprintf(f,"\\hline\n");
  }

  void endTable()
  {
    fprintf(f,"\\hline");
    
    fprintf(f,"\\end{tabular}\n");
  }

  void beginRow()
  {
  }

  void endRow()
  {
    fprintf(f,"\\\\\n");
  }

  void beginEntry()
  {
  }

  void endEntry()
  {
    fprintf(f," & ");
  }

  void printString(const char *s)
  {
    fprintf(f,"%s",s);
  }

  void hline()
  {
    fprintf(f,"");
  }

  void flush()
  {
    fflush(f);
  }

  void newLine()
  {
  }
};
*/

class HTMLprinter:public printer
{
  FILE *f;
  void begin()
  {
    f=fopen("performance.html","w");

    fprintf(f,"<html>\n<head>\n</head>\n<body>\n");
  }

  void end()
  {
    fprintf(f,"</body>\n</html>\n");
    fclose(f);
  }

  void beginTable()
  {
    fprintf(f,"<table>\n");
  }

  void endTable()
  {
    fprintf(f,"</table>\n");
  }

  void beginRow()
  {
    fprintf(f,"<tr>\n");
  }

  void endRow()
  {
    fprintf(f,"</tr>\n");
  }

  void beginEntry()
  {
    fprintf(f,"<td>\n");
  }

  void endEntry()
  {
    fprintf(f,"</td>\n");
  }

  void printString(const char *s)
  {
    fprintf(f,"%s",s);
  }

  void hline()
  {
    fprintf(f,"");
  }

  void flush()
  {
    fflush(f);
  }

  void newLine()
  {
    fprintf(f,"<br>\n");
  }
};

//--------------------------------------------------------------
// Algorithm / Ideal specifier classes
//--------------------------------------------------------------

class algorithm
{
public:
  testresult result;
  int level;
  char *options;
  char *name;
};

class ideal
{
public:
  testresult result;
  int level;
  char *representation;
  char *name;
};

typedef list<algorithm> algorithmList;
typedef list<ideal> idealList;
algorithmList algorithms;
idealList ideals;

//--------------------------------------------------------------
// Input file parser
//--------------------------------------------------------------

bool parseLine(FILE *f, char* &tag, char* &value)
{
  char s[1024];
  int c;

  do
    {
      c=fgetc(f);
    }
  while(c==' ' || c=='\t' || c=='\n');
  ungetc(c,f);

  int t=0;
  do
    {
      s[t++]=c=fgetc(f);
    }
  while(c!=EOF && c!=':');
  if(c==EOF)return false;
  s[t++]=0;
  tag=(char*)malloc(sizeof(char)*t);
  sprintf(tag,"%s",s);

  do
    {
      c=fgetc(f);
    }
  while(c==' ');
  ungetc(c,f);

  t=0;
  do
    {
      s[t++]=c=fgetc(f);
    }
  while(c!=EOF && c!='\n');
  if(c==EOF)return false;
  s[t++]=0;
  value=(char*)malloc(sizeof(char)*t);
  sprintf(value,"%s",s);

  return true;
}

void parseFile(FILE *f=stdin)
{
  char *tag,*value;

  while(parseLine(f,tag,value))
    {
      if(strcmp(tag,"algorithm:")==0)
	{
	  algorithm a;
	  parseLine(f,tag,value);
	  assert(strcmp(tag,"name:")==0);
	  a.name=value;
	  parseLine(f,tag,value);
	  assert(strcmp(tag,"options:")==0);
	  a.options=value;
	  parseLine(f,tag,value);
	  assert(strcmp(tag,"level:")==0);
	  a.level=atoi(value);
	  algorithms.push_back(a);
	}
      else if(!strcmp(tag,"option:")==0)
	{
	  ideal i;
	  parseLine(f,tag,value);
	  assert(strcmp(tag,"name:")==0);
	  i.name=value;
	  parseLine(f,tag,value);
	  assert(strcmp(tag,"representation:")==0);
	  i.representation=value;
	  parseLine(f,tag,value);
	  assert(strcmp(tag,"level:")==0);
	  i.level=atoi(value);
	  ideals.push_back(i);
	}
      else assert("PARSE ERROR"==0);
    }
}


//--------------------------------------------------------------
// Procedures for cpuinfo
//--------------------------------------------------------------

char *cpuInfoSearch(char *str)
{
  static char s[256];

  FILE *f=fopen("/proc/cpuinfo","r");
  if(f==0)return "Unknown";

  while(!feof(f))
    {
      fscanf(f,"%s",s);
      if(strcmp(str,s)==0)
	{
	  int c;
	  do
	    {
	      c=fgetc(f);
	    }
	  while(c!=EOF && c!=':');
	  if(c==EOF)
	    {
	      fclose(f);
	      return "Unknown";
	    }
	  fgets(s,255,f);
	  fclose(f);
	  return s;
	}
    }

  fclose(f);
  return "Unknown";
}

char *cpuName()
{
  return cpuInfoSearch("name");
}

char *cpuFreq()
{
  return cpuInfoSearch("MHz");
}


//--------------------------------------------------------------
// procedures for doing the tests
//--------------------------------------------------------------
/*
int system2(char *s)
{
  testresult t=testresult_new();

  testresult_write(t);

  return system(s);
}
*/

printer *thePrinter;

void singleTest(algorithm &a, ideal &i)
{
  char s[1024];

  FILE *f=fopen("cats_inputfile","w");
  fprintf(f,"%s\n",i.representation);
  fclose(f);

  sprintf(s,"./cats -i cats_inputfile %s",a.options);
  system(s);

  a.result=testresult_read("output_results");
  if(a.result.floatingpointstatus==0)
    {
      if(i.result.dim_d==-1)i.result.dim_d=a.result.dim_d;
      if(i.result.dim_n==-1)i.result.dim_n=a.result.dim_n;
      if(i.result.kerdim==-1)i.result.kerdim=a.result.kerdim;
      if(i.result.size_grevlex==-1)i.result.size_grevlex=a.result.size_grevlex;
      if(i.result.size_lex==-1)i.result.size_lex=a.result.size_lex;
      if(i.result.size_gfan==-1)i.result.size_gfan=a.result.size_gfan;
      if(i.result.nedges_gfan==-1)i.result.nedges_gfan=a.result.nedges_gfan;
    }
}

void intEntry(printer &thePrinter, int i)
{
  char s[32];

  sprintf(s,"%i",i);
  if(i==-1)
    thePrinter.entry("-");
  else
    thePrinter.entry(s);
}

void floatEntry(printer &thePrinter, float i)
{
  char s[32];

  sprintf(s,"%8.2f",i);
  if(i<0)
    thePrinter.entry("-");
  else
    thePrinter.entry(s);
}

void runTests(printer &thePrinter)
{
  char s[1024];

  thePrinter.beginTable();
  thePrinter.beginRow();
  thePrinter.entry("A");
  thePrinter.entry("d");
  thePrinter.entry("n");
  thePrinter.entry("kd");
  thePrinter.entry("vertices");
  thePrinter.entry("edges");
  for(algorithmList::iterator a=algorithms.begin();a!=algorithms.end();a++)
    {
      thePrinter.beginEntry();
      thePrinter.printString(a->name);
      thePrinter.endEntry();
    }
  thePrinter.endRow();
  thePrinter.hline();

  for(idealList::iterator i=ideals.begin();i!=ideals.end();i++)
    {
      i->result=testresult_new();
      for(algorithmList::iterator a=algorithms.begin();a!=algorithms.end();a++)
	{
	  a->result=testresult_new();
	  if(a->level>=i->level)
	    singleTest(*a,*i);
	} 
      thePrinter.beginRow();
      thePrinter.entry(i->name);
      intEntry(thePrinter,i->result.dim_d);
      intEntry(thePrinter,i->result.dim_n);
      intEntry(thePrinter,i->result.kerdim);
      intEntry(thePrinter,i->result.size_gfan);
      intEntry(thePrinter,i->result.nedges_gfan);
      for(algorithmList::iterator a=algorithms.begin();a!=algorithms.end();a++)
	{
	  floatEntry(thePrinter,a->result.time/1000.0f);
	}
      thePrinter.endRow();
      thePrinter.flush();
    }
  thePrinter.endTable();
}

//--------------------------------------------------------------
// Main
//--------------------------------------------------------------

int main()
{
  stdin=freopen("performance_list","r",stdin);
  parseFile();

  thePrinter = new HTMLprinter();
  thePrinter->begin();
  thePrinter->printString("Host: ");
  thePrinter->printString(getenv("HOST"));
  thePrinter->printString("  ");
  thePrinter->printString("CpuType: ");
  thePrinter->printString(cpuName());
  thePrinter->printString(cpuFreq());
  thePrinter->printString("MHz");
  thePrinter->newLine();

  runTests(*thePrinter);
      
  thePrinter->end();

  return 0;
}
